#!/usr/bin/env python

from urlparse import urlparse
from . import env


def from_heroku_envvars(config):
        var_map = {
            # SQL-Alchemy
            'DATABASE_URL': 'SQLALCHEMY_DATABASE_URI',

            # newer-style
            'HEROKU_POSTGRESQL_ORANGE_URL': 'SQLALCHEMY_DATABASE_URI',

            # Celery w/ RabbitMQ
            'BROKER_URL': 'RABBITMQ_URL',

            'REDISTOGO_URL': 'REDIS_URL',
            'MONGOLAB_URI': 'MONGO_URI',
            'MONGOHQ_URL': 'MONGO_URI',
            'CLOUDANT_URL': 'COUCHDB_URL',

            'MEMCACHIER_SERVERS': 'CACHE_MEMCACHED_SERVERS',
            'MEMCACHIER_USERNAME': 'CACHE_MEMCACHED_USERNAME',
            'MEMCACHIER_PASSWORD': 'CACHE_MEMCACHED_PASSWORD',
        }

        var_list = [
            # Sentry
            'SENTRY_DSN',

            # Exceptional
            'EXCEPTIONAL_API_KEY',

            # Flask-GoogleFed
            'GOOGLE_DOMAIN',

            # Mailgun
            'MAILGUN_API_KEY', 'MAILGUN_SMTP_LOGIN', 'MAILGUN_SMTP_PASSWORD',
            'MAILGUN_SMTP_PORT', 'MAILGUN_SMTP_SERVER',

            # SendGrid
            'SENDGRID_USERNAME', 'SENDGRID_PASSWORD'
        ]

        # import the relevant envvars
        env.from_envvars(config, envvars=var_list, as_json=False)
        env.from_envvars(config, envvars=var_map, as_json=False)

        # fix up configuration
        if 'MAILGUN_SMTP_SERVER' in config:
            config['SMTP_SERVER'] = config['MAILGUN_SMTP_SERVER']
            config['SMTP_PORT'] = config['MAILGUN_SMTP_PORT']
            config['SMTP_LOGIN'] = config['MAILGUN_SMTP_LOGIN']
            config['SMTP_PASSWORD'] = config['MAILGUN_SMTP_PASSWORD']
            config['SMTP_USE_TLS'] = True
        elif 'SENDGRID_USERNAME' in config:
            config['SMTP_SERVER'] = 'smtp.sendgrid.net'
            config['SMTP_PORT'] = 25
            config['SMTP_LOGIN'] = config['SENDGRID_USERNAME']
            config['SMTP_PASSWORD'] = config['SENDGRID_PASSWORD']
            config['SMTP_USE_TLS'] = True

        # convert to Flask-Mail specific configuration
        if 'MAILGUN_SMTP_SERVER' in config or\
           'SENDGRID_PASSWORD' in config:

            config['MAIL_SERVER'] = config['SMTP_SERVER']
            config['MAIL_PORT'] = config['SMTP_PORT']
            config['MAIL_USE_TLS'] = config['SMTP_USE_TLS']
            config['MAIL_USERNAME'] = config['SMTP_LOGIN']
            config['MAIL_PASSWORD'] = config['SMTP_PASSWORD']

        # for backwards compatiblity, redis:
        if 'REDIS_URL' in config:
            url = urlparse(config['REDIS_URL'])
            config['REDIS_HOST'] = url.hostname
            config['REDIS_PORT'] = url.port
            config['REDIS_PASSWORD'] = url.password
            # FIXME: missing db#?

        if 'MONGO_URI' in config:
            url = urlparse(config['MONGO_URI'])
            config['MONGODB_USER'] = url.username
            config['MONGODB_PASSWORD'] = url.password
            config['MONGODB_HOST'] = url.hostname
            config['MONGODB_PORT'] = url.port
            config['MONGODB_DB'] = url.path[1:]



#!/usr/bin/env python

import os

from . import env, heroku


class AppConfig(object):
    def __init__(self, app=None, *args, **kwargs):
        if app:
            self.init_app(app, *args, **kwargs)

    def init_app(self, app,
                 configfile=None, envvar=True, default_settings=True,
                 from_envvars='json', from_envvars_prefix=None):

        if from_envvars_prefix is None:
            from_envvars_prefix = app.name.upper() + '_'

        if default_settings is True:
            try:
                app.config.from_object(app.name + '.default_config')
            except ImportError:
                # ignore import errors if settings were autogenerated
                pass
        elif default_settings:
            app.config.from_object(default_settings)

        # load supplied configuration file
        if configfile:
            app.config.from_pyfile(configfile)

        # load configuration file from environment
        if envvar is True:
            envvar = app.name.upper() + '_CONFIG'

        if envvar and envvar in os.environ:
            app.config.from_envvar(envvar)

        # load environment variables
        if from_envvars:
            env.from_envvars(app.config, from_envvars_prefix,
                             as_json=('json' == from_envvars))

        # register extension
        app.extensions = getattr(app, 'extensions', {})
        app.extensions['appconfig'] = self


class HerokuConfig(AppConfig):
    def init_app(self, app, *args, **kwargs):
        super(HerokuConfig, self).init_app(app, *args, **kwargs)

        heroku.from_heroku_envvars(app.config)



import argparse
import sys


def main_flaskdev():
    try:
        import importlib
    except ImportError:
        print ('You do not have importlib installed. Please install a '
               'backport for versions < 2.7/3.1 of it first.')
        sys.exit(1)

    parser = argparse.ArgumentParser(
        description='Imports a module passed on the commandline, instantiates '
                    'an app by calling imported_module.create_app() with an '
                    'optional configuration file and runs it in debug mode.',
    )
    parser.add_argument('module_name', help='Name of the module to import.')
    parser.add_argument('-c', '--configfile',
                        help='Configuration file to pass as the first '
                        'parameter to create_app.')

    args = parser.parse_args()
    mod = importlib.import_module(args.module_name)
    mod.create_app(args.configfile).run(debug=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='flask-appconfig',
    version='0.5',
    description=('Configures Flask applications in a canonical way. Also auto-'
                 'configures Heroku. Aims to standardize configuration.'),
    long_description=read('README.rst'),
    author='Marc Brinkmann',
    author_email='git@marcbrinkmann.de',
    url='http://github.com/mbr/flask-appconfig',
    license='MIT',
    packages=find_packages(exclude=['test']),
    install_requires=['flask'],
    entry_points={
        'console_scripts': [
            'flaskdev = flask_appconfig.cmd:main_flaskdev',
        ],
    }
)
